package com.engine.salary.mapper.auth;

import com.engine.salary.entity.auth.dto.AuthRoleEmpDTO;
import com.engine.salary.entity.auth.po.AuthRoleEmpPO;
import org.apache.ibatis.annotations.Param;

import java.util.Collection;
import java.util.List;

public interface AuthRoleEmpMapper {

	/**
     * 查询所有记录
     *
     * @return 返回集合，没有返回空List
     */
	List<AuthRoleEmpPO> listAll();
    
    /**
     * 条件查询
     *
     * @return 返回集合，没有返回空List
     */
    List<AuthRoleEmpPO> listSome(AuthRoleEmpPO authRoleEmp);
    

	/**
     * 根据主键查询
     *
     * @param id 主键
     * @return 返回记录，没有返回null
     */
	AuthRoleEmpPO getById(Long id);
	
	/**
     * 新增，忽略null字段
     *
     * @param authRoleEmp 新增的记录
     * @return 返回影响行数
     */
	int insertIgnoreNull(AuthRoleEmpPO authRoleEmp);
    
    /**
     * 批量插入
     *
     * @param authRoleEmp
     */
    void batchInsert(@Param("collection") List<AuthRoleEmpPO> authRoleEmp);
	
	/**
     * 修改，修改所有字段
     *
     * @param authRoleEmp 修改的记录
     * @return 返回影响行数
     */
	int update(AuthRoleEmpPO authRoleEmp);
	
	/**
     * 修改，忽略null字段
     *
     * @param authRoleEmp 修改的记录
     * @return 返回影响行数
     */
	int updateIgnoreNull(AuthRoleEmpPO authRoleEmp);
	
	/**
     * 删除记录
     *
     * @param authRoleEmp 待删除的记录
     * @return 返回影响行数
     */
	int delete(AuthRoleEmpPO authRoleEmp);
    
    /**
     * 批量删除记录
     * @param ids 主键id集合
     */
    void deleteByIds(@Param("ids") Collection<Long> ids);

	/**
	 * 根据角色id删除
	 * @param roleId roleId
	 */
	void deleteByRoleId(Long roleId);

    List<AuthRoleEmpDTO> getByRoleId(Long roleId);
}